(function ($) {
    "use strict";
    // ==================== #region handle request ==================== //
    class PolyHandleRequest {
        /**
         * Handles the execution of a function with a specified list of elements during the ajaxComplete event.
         * @param {string} callbackFunction The name of the function to execute for each element in argsArray during the ajaxComplete event.
         * @param {array} argsArray An array of elements to iterate over and invoke in the callbackFunction. Defaults to null, which calls callbackFunction directly.
         */
        static ajaxComplete(callbackFunction, argsArray = null) {
            $(document).ajaxComplete(function (event, xhr, settings) {
                if (argsArray != null) {
                    argsArray.forEach(args => {
                        callbackFunction(...args);
                    });
                } else {
                    callbackFunction();
                }
            });
        }
    }
    // ==================== #endregion handle request ==================== //
    window.PolyHandleRequest = PolyHandleRequest;
})(jQuery);